@@ -69,11 +69,18 @@ year_quarter <- function(x){
 YM <- function(length = 0L){
   new_year_month(integer(length))
 }
+#' @rdname year_month
+#' @export
+year_month_decimal <- function(x){
+  m <- ( x * 12 ) - (1970 * 12)
+  new_year_month(m)
+}
 new_year_month <- function(x){
   check_is_num(x)
   class(x) <- "year_month"
   x
 }
+#' @export
 `[.year_month` <- function(x, ..., drop = TRUE){
   cl <- oldClass(x)
   class(x) <- NULL
@@ -81,6 +88,7 @@ new_year_month <- function(x){
   class(val) <- cl
   val
 }
+#' @export
 `[[.year_month` <- function(x, ..., drop = TRUE){
   cl <- oldClass(x)
   class(x) <- NULL
@@ -88,6 +96,7 @@ new_year_month <- function(x){
   class(val) <- cl
   val
 }
+#' @export
 `+.year_month` <- function(e1, e2){
   out <- unclass(e1) + unclass(e2)
   both_year_month <- inherits(e1, "year_month") && inherits(e2, "year_month")
@@ -96,6 +105,7 @@ new_year_month <- function(x){
   }
   out
 }
+#' @export
 `-.year_month` <- function(e1, e2){
   out <- unclass(e1) - unclass(e2)
   both_year_month <- inherits(e1, "year_month") && inherits(e2, "year_month")
@@ -104,9 +114,11 @@ new_year_month <- function(x){
   }
   out
 }
+#' @exportS3Method base::c
 `c.year_month` <- function(...){
   new_year_month(do.call(c, lapply(list(...), unclass)))
 }
+#' @exportS3Method base::print
 print.year_month <- function(x, max = NULL, ...){
   z <- unclass(x)
   if (length(z) == 0){
@@ -131,6 +143,7 @@ print.year_month <- function(x, max = NULL, ...){
   }
   invisible(x)
 }
+#' @exportS3Method base::as.character
 as.character.year_month <- function(x, ...){
   x <- unclass(x)
   if (length(x) == 0){
@@ -142,6 +155,7 @@ as.character.year_month <- function(x, ...){
     stringr::str_c(y, mf, sep = " ")
   }
 }
+#' @exportS3Method base::format
 format.year_month <- function(x, ...){
   x <- unclass(x)
   if (length(x) == 0){
@@ -153,21 +167,25 @@ format.year_month <- function(x, ...){
     format(stringr::str_c(y, mf, sep = " "), ...)
   }
 }
+#' @exportS3Method base::unique
 unique.year_month <- function(x, incomparables = FALSE, ...){
   new_year_month(unique.default(x, incomparables = incomparables, ...))
 }
+#' @exportS3Method base::as.Date
 as.Date.year_month <- function(x, ...){
   x <- unclass(x)
   y <- x %/% 12L
   m <- (x %% 12L) + 1L
   lubridate::make_date(year = 1970L + y, month = m, day = 1L)
 }
+#' @exportS3Method base::as.POSIXct
 as.POSIXct.year_month <- function(x, tz = "", ...){
   x <- unclass(x)
   y <- x %/% 12L
   m <- (x %% 12L) + 1L
   lubridate::make_datetime(year = 1970L + y, month = m, day = 1L)
 }
+#' @exportS3Method base::as.POSIXlt
 as.POSIXlt.year_month <- function(x, tz = "", ...){
   as.POSIXlt(as.POSIXct(x, tz = tz))
 }
@@ -181,6 +199,13 @@ new_year_quarter <- function(x){
 YQ <- function(length = 0L){
   new_year_quarter(integer(length))
 }
+#' @rdname year_month
+#' @export
+year_quarter_decimal <- function(x){
+  q <- ( x * 4 ) - (1970 * 4)
+  new_year_quarter(q)
+}
+#' @export
 `[.year_quarter` <- function(x, ..., drop = TRUE){
   cl <- oldClass(x)
   class(x) <- NULL
@@ -188,6 +213,7 @@ YQ <- function(length = 0L){
   class(val) <- cl
   val
 }
+#' @export
 `[[.year_quarter` <- function(x, ..., drop = TRUE){
   cl <- oldClass(x)
   class(x) <- NULL
@@ -195,6 +221,7 @@ YQ <- function(length = 0L){
   class(val) <- cl
   val
 }
+#' @export
 `+.year_quarter` <- function(e1, e2){
   out <- unclass(e1) + unclass(e2)
   both_year_quarter <- inherits(e1, "year_quarter") && inherits(e2, "year_quarter")
@@ -203,6 +230,7 @@ YQ <- function(length = 0L){
   }
   out
 }
+#' @export
 `-.year_quarter` <- function(e1, e2){
   out <- unclass(e1) - unclass(e2)
   both_year_quarter <- inherits(e1, "year_quarter") && inherits(e2, "year_quarter")
@@ -211,9 +239,11 @@ YQ <- function(length = 0L){
   }
   out
 }
+#' @exportS3Method base::c
 `c.year_quarter` <- function(...){
   new_year_quarter(do.call(c, lapply(list(...), unclass)))
 }
+#' @exportS3Method base::print
 print.year_quarter <- function(x, max = NULL, ...){
   z <- unclass(x)
   if (length(z) == 0){
@@ -238,6 +268,7 @@ print.year_quarter <- function(x, max = NULL, ...){
   }
   invisible(x)
 }
+#' @exportS3Method base::as.character
 as.character.year_quarter <- function(x, ...){
   x <- unclass(x)
   if (length(x) == 0){
@@ -249,6 +280,7 @@ as.character.year_quarter <- function(x, ...){
     stringr::str_c(y, qf, sep = " ")
   }
 }
+#' @exportS3Method base::format
 format.year_quarter <- function(x, ...){
   x <- unclass(x)
   if (length(x) == 0){
@@ -260,40 +292,49 @@ format.year_quarter <- function(x, ...){
     format(stringr::str_c(y, qf, sep = " "), ...)
   }
 }
+#' @exportS3Method base::unique
 unique.year_quarter <- function(x, incomparables = FALSE, ...){
   new_year_quarter(unique.default(x, incomparables = incomparables, ...))
 }
-
+#' @exportS3Method base::as.Date
 as.Date.year_quarter <- function(x, ...){
   x <- unclass(x)
   y <- 1970L + (x %/% 4L)
   m <- 3L * (x %% 4L) + 1L
   lubridate::make_date(year = y, month = m, day = 1L)
 }
+#' @exportS3Method base::as.POSIXct
 as.POSIXct.year_quarter <- function(x, tz = "", ...){
   x <- unclass(x)
   y <- 1970L + (x %/% 4L)
   m <- 3L * (x %% 4L) + 1L
   lubridate::make_datetime(year = y, month = m, day = 1L)
 }
+#' @exportS3Method base::as.POSIXlt
 as.POSIXlt.year_quarter <- function(x, tz = "", ...){
   as.POSIXlt(as.POSIXct(x, tz = tz))
 }
+#' @exportS3Method base::rep_len
 rep_len.year_month <- function(x, length.out){
   x[rep_len(seq_along(x), length.out = length.out)]
 }
+#' @exportS3Method base::rep.int
 rep.int.year_month <- function(x, ...){
   x[rep.int(seq_along(x), ...)]
 }
+#' @exportS3Method base::rep
 rep.year_month <- function(x, ...){
   x[rep(seq_along(x), ...)]
 }
+#' @exportS3Method base::rep_len
 rep_len.year_quarter <- function(x, length.out){
   x[rep_len(seq_along(x), length.out = length.out)]
 }
+#' @exportS3Method base::rep.int
 rep.int.year_quarter <- function(x, ...){
   x[rep.int(seq_along(x), ...)]
 }
+#' @exportS3Method base::rep
 rep.year_quarter <- function(x, ...){
   x[rep(seq_along(x), ...)]
 }
