@@ -1,8 +1,4 @@
-#include <cpp11.hpp>
-#include <Rinternals.h>
-
-#define R_NO_REMAP
-
+#include "timeplyr_cpp.h"
 
 // Row numbers by group using order of groups and group sizes
 [[cpp11::register]]
@@ -21,25 +17,21 @@ SEXP cpp_row_id(SEXP order, SEXP group_sizes, bool ascending){
   }
   if (ascending){
     int init = 0;
-    for (int i = 0; i < n; i++){
-      if (i > (running_group_size - 1)){
-        ++j;
+    for (int i = 0; i < n; ++i){
+      if (i >= running_group_size){
         init = 0;
-        running_group_size += p_group_sizes[j];
+        running_group_size += p_group_sizes[++j];
       }
-      ++init;
-      p_out[p_o[i] - 1] = init;
+      p_out[p_o[i] - 1] = ++init;
     }
   } else {
     int init = running_group_size + 1;
-    for (int i = 0; i < n; i++){
-      if (i > (running_group_size - 1)){
-        ++j;
-        init = p_group_sizes[j] + 1;
+    for (int i = 0; i < n; ++i){
+      if (i >= running_group_size){
+        init = p_group_sizes[++j] + 1;
         running_group_size += p_group_sizes[j];
       }
-      --init;
-      p_out[p_o[i] - 1] = init;
+      p_out[p_o[i] - 1] = --init;
     }
   }
   Rf_unprotect(1);
